Public Module Common

    'Structure declarations
    Public Structure JprStructure   'Necessary JPR data (in the context of this application)
        Public nm As String         'Name of the Chart
        Public sc As Double         'Scale of the Chart
        Public sr As Double         'Resolution of the Chart
        Public it As String         'Type of image of the Chart
        Public dm As String         'Geographic datum of the Chart
        Public st As String         'Datum shift (latitude)
        Public sn As String         'Datum shift (longitude)
        Public pr As String         'Projection of the Chart
        Public zn As String         'UTM zone of the Chart
        Public pp As String         'Central meridian of the Chart
        Public p1 As String         'Latitude of the origin of the Chart
        Public p2 As String         'Scalefactor of the Chart
        Public p3 As String         'False Easting of the Chart
        Public p4 As String         'Fales Northing of the Chart
        Public p5 As String         'First standard parallel of the Chart
        Public p6 As String         'Second standard parallel of the Chart
        Public gs As String         'Type of coordinate/grid system (not a formal JPR variable)
        Public gu As String         'Unit of coordinate/grid system (not a formal JPR variable/only meter)
        Public cu As String         'Unit of contour lines (height)
        Public ds As Double         'Size of pixel in meters (not a formal JPR variable)
        Public ps As Double         'Size of pixel in units of the coordinate system (not a formal JPR variable)
        Public cd As String         'Description of colordepth if image (not a formal JPR variable)
    End Structure

    Public Structure FileNameStructure
        Public LongName As String       'Filename with path
        Public Name As String           'Filename without path
        Public ShortName As String      'Filename without extension
        Public Path As String           'Path to file
        Public Ext As String            'Extension of filename
    End Structure

    Public Structure Coordinate         'Referenced Pixel with pixel coordinates, geographical coordinates and easting/northing
        Public x As String              'Pixel X
        Public y As String              'Pixel Y
        Public lat As String            'Latitude
        Public lon As String            'Longitude
        Public east As String           'Easting
        Public north As String          'Northing
        Public Valid As Integer         'Application internal variable used for validation
    End Structure

    Public Structure ResStructure           'Gdal data about Resolution
        Public ResolutionX As Single        'Horizontal resolotion 
        Public ResolutionY As Single        'Vertical resolution
        Public ResolutionUnit As Integer    'Unit used for resolution (2=DPI/3=Dot per cm)
        Public PixelSizeX As Double         'Horizontal size of a pixel in reality (in meter)
        Public PixelSizeY As Double         'Vertical size of a pixel in reality (in meter)
    End Structure

    Public Structure ImageDataStructure
        Public Exists As Boolean            'Indication the image file exits
        Public Valid As Boolean             'Indication the image data could extracted
        Public Usable As Boolean            'Indication the image colordepth is compatible to Memory-Map
        Public PixelSizeX As Double         'Horizontal size of a pixel in reality (in meter)
        Public PixelSizeY As Double         'Vertical size of a pixel in reality (in meter)
        Public ResolutionX As Single        'Horizontal resolotion 
        Public ResolutionY As Single        'Vertical resolution
        Public ColorDepth As String         'Description of teh colordepth
    End Structure

    Structure BlockStructure                'Structure used when clipping the WKT in to Blocks (to prevent double values for variables
        Public FirstLine As Integer
        Public LastLine As Integer
    End Structure

    Public TempImage As Image

    'Variable declarations
    Public ReadOnly C34 As String = Chr(34)                                 'Variable represents the character 34 ("), used building commandline strings 
    Public ReadOnly EndLine As String = Chr(13) + Chr(10)                   'Variable represents the end a line of text
    Public ReadOnly Version As String = "1.0.0"                             'variable represents the version number
    Public ReadOnly VersionName As String = " (Pontresina)"                    'variable represents the version name (nickname)
    Public ReadOnly AppName As String = "Maps for Memory-Map " & Version    'Variable represents name op the application used in Label of forms
    Public BatFile As FileNameStructure                                     'Variable represents path to OSgeo4W.bat

    Public ImageWidth As Long                                               'Width of the image im pixel
    Public ImageHeight As Long                                              'Height of the image in pixel
    Public ImageDiagonal As Double                                          'The length of image diagonal

    'Variables decalrations for printing
    Public StringToPrint As String                                          'Variable represents text to be printed (by DocumentToPrint)
    Public PageToPrint(1) As String                                         'Array represents pages to print (will be redimensioned)
    Public NumberOfPages As Integer                                         'Variable represents the number of pages in PageToPrint array (0 is the first)
    Public PagePointer As Integer                                           'Variable represents the pointer used with array PageToPrint
    Public LineInString(1) As String                                        'Array represents lines in a string (will be redimensioned)
    Public NumberOfLines As Integer                                         'Variable represents the number of lines in LineInString array (0 is the first)
    Public LinePointer As Integer                                           'Variable represents the pointer used with LineInString array

    'Control declarations
    Public SelectFile As New System.Windows.Forms.OpenFileDialog()      'Creating a File (select) dialog object
    Public SelectPrinter As New System.Windows.Forms.PrintDialog()      'Creating a printer (select) dialog object

    'Funtcion SplitFileName
    '- Splits a string representing a filename with path into five "sub" variables (FileNameStructure)
    '- Used throughout the application
    Public Function SplitFileName(FileNameString As String) As FileNameStructure
        '- Declarations
        Dim Result As FileNameStructure
        Dim Pointer As Integer
        Dim StringLen As Integer

        '- Clearing all values 
        Result.LongName = ""
        Result.Name = ""
        Result.ShortName = ""
        Result.Path = ""
        Result.Ext = ""

        Result.LongName = FileNameString

        '- Retrieving values for .Path and .Name
        If InStr(FileNameString, "\", CompareMethod.Binary) > 0 Then    'excluding filename without path
            StringLen = Len(FileNameString)
            For Pointer = Len(FileNameString) To 0 Step -1              'Walking back from last character until \
                If Strings.Mid(FileNameString, Pointer, 1) = "\" Then
                    Result.Path = Strings.Mid(FileNameString, 1, Pointer)
                    Result.Name = Strings.Mid(FileNameString, Pointer + 1, Len(FileNameString) - Pointer)
                    Exit For
                End If
            Next Pointer
        Else
            Result.Name = FileNameString                                'Case no \ found
        End If

        '- Retrieving for values for .ShortName abnd .Ext
        StringLen = Len(Result.Name)
        If InStr(FileNameString, ".", CompareMethod.Binary) > 0 Then    'excluding filename without path
            For Pointer = Len(Result.Name) To 0 Step -1                 'Walking back from last character until .
                If Mid(Result.Name, Pointer, 1) = "." Then
                    Result.ShortName = Strings.Mid(Result.Name, 1, Pointer - 1)
                    Result.Ext = Strings.Mid(Result.Name, Pointer + 1, Len(Result.Name) - Pointer)
                    Exit For
                End If
            Next Pointer
        Else
            Result.ShortName = Result.Name                              'Çase no .(dot) found
        End If

        Return Result

    End Function

    'Function TranslateDatum harmonized all kind of datum names to names Memory-Map accepts
    '- The values were collected during the internet search for usable test examples.
    Public Function TranslateDatum(Datum As String)
        Dim Result As String

        Datum = Trim(LCase(Datum))

        Select Case Datum
            Case "north american 1927 (canada mean)",
                 "north american 1927",
                 "north american datum 1927",
                 "nad27", "nad 27",
                 "north american 1927 (continental us)",
                 "nad27 canada", "nad27 conus"
                Result = "nad27"
            Case "north american 1983", "north american datum 1983", "nad83", "nad 83"
                Result = "nad83"
            Case "wgs84", "wgs 84", "world geodetic system 1984",
                 "etrs89", "etrs 89", "european terrestrial reference system 1989",
                 "itrf92", "itrf 92", "itrf2008", "itrf 2008",
                 "gda94", "gda 94", "geocentric datum of australia 1994"
                Result = "wgs84"
            Case "osgb", "osgb 1936", "osgb1936", "osgb36", "osgb 36"
                Result = "osgb"
            Case Else
                Result = Datum
        End Select

        '- Returning Result
        Return Result

    End Function

    'Function TranslateProjection harmonized all kind of projection names to names Memory-Map accepts.
    '- The values were collected during the internet search for usable test examples.
    Public Function TranslateProjection(Projection As String, Zone As String)
        Dim Result As String

        Projection = LCase(Projection)

        Select Case Projection
            Case "merc", "mercator",
                 "popular visualisation pseudo mercator"
                Result = "mercator"
            Case "tmerc", "transverse mercator"
                If Zone <> "" Then
                    Result = "utm"
                Else
                    Result = "transverse mercator"
                End If
            Case "lambert conformal conic", "lambert", "lambert cc", "lambert conic conformal (2sp)"
                Result = "lambert conformal conic"
            Case "(utm) universal transverse mercator", "utm", "universal transverse mercator"
                Result = "utm"
            Case Else
                Result = Projection
        End Select

        '- Returning Result
        Return Result

    End Function

    'Function ClearResolution reset all filename values to ""
    Public Function ClearFileName()
        '- Declaration
        Dim Result As FileNameStructure

        '- Resetting values
        Result.LongName = ""
        Result.Name = ""
        Result.ShortName = ""
        Result.Path = ""
        Result.Ext = ""

        '- Returning Result
        Return Result

    End Function

    'Function ClearResolution reset all resolution values to zero
    Public Function ClearResolution()
        '- Declaration
        Dim Result As ResStructure

        '- Resetting values
        Result.ResolutionX = 0
        Result.ResolutionY = 0
        Result.ResolutionUnit = 0
        Result.PixelSizeX = 0
        Result.PixelSizeY = 0

        '- Returning Result
        Return Result

    End Function

    'Function ClearResolution reset all coordinate values to ""/zero
    Function ClearCoordinate()
        '- Declaration
        Dim Result As Coordinate

        '- Resetting values
        Result.x = ""
        Result.y = ""
        Result.lat = ""
        Result.lon = ""
        Result.east = ""
        Result.north = ""
        Result.Valid = 0

        '- Returning Result
        Return Result

    End Function

    'Function ClearJprData reset all JPR values to 
    Public Function ClearJprData()
        '- Declaration
        Dim Result As JprStructure

        '- Resetting values
        Result.nm = ""
        Result.it = ""
        Result.sc = 0
        Result.sr = 0
        Result.ds = 0
        Result.ps = 0
        Result.dm = ""
        Result.st = ""
        Result.sn = ""
        Result.pr = ""
        Result.zn = ""
        Result.pp = ""
        Result.p1 = ""
        Result.p2 = ""
        Result.p3 = ""
        Result.p4 = ""
        Result.p5 = ""
        Result.p6 = ""
        Result.gs = ""
        Result.gu = ""
        Result.cu = ""
        Result.cd = ""

        '- Returning Result
        Return Result

    End Function

    'Function CommaToDot converts, in a string representing as number, the comma (,) 
    'as a seperator character into a dot (.).
    Public Function CommaToDot(CommaString As String) As String
        'Declarations
        Dim TempText As String
        Dim ChrPointer As Integer

        '- Moving character by character form 'CommaString' to 'TempText'
        '  unless it is a comma (,). In this case the comma will be replaced by a dot (.).
        TempText = ""
        For ChrPointer = 1 To Len(CommaString)
            If Mid(CommaString, ChrPointer, 1) <> "," Then
                TempText = TempText + Mid(CommaString, ChrPointer, 1)
            Else
                TempText = TempText & "."
            End If
        Next

        '- Returning string to function and ending
        Return TempText

    End Function

    'Function CvtLatLonStr converts the string expression, representing a Latitude or longitude, 
    '  (in degree, minutes, secondes and sign) in a string expression, representing a Latitude 
    '   or longitude in decimal degree.
    Public Function CvtLatLonStr(LatLonStr As String) As String
        '- Declarations
        Dim ValDeg, ValMin, ValSec, ValSign As Double
        Dim TempText1, TempText2 As String
        Dim ChrPointer As Integer

        '- Extracting value degrees (ValDeg)
        TempText1 = LatLonStr
        ChrPointer = InStr(TempText1, "d", CompareMethod.Binary)
        TempText2 = Trim(Mid(TempText1, 1, ChrPointer - 1))
        ValDeg = Val(TempText2)

        '- Extracting value minutes (ValMin)
        TempText1 = Mid(TempText1, ChrPointer + 1, Len(TempText1) - ChrPointer)
        ChrPointer = InStr(TempText1, "'", CompareMethod.Binary)
        TempText2 = Trim(Mid(TempText1, 1, ChrPointer - 1))
        ValMin = Val(TempText2)

        '- Extracting value Secondes (ValSec)
        TempText1 = Mid(TempText1, ChrPointer + 1, Len(TempText1) - ChrPointer)
        ChrPointer = InStr(TempText1, Chr(34), CompareMethod.Binary)
        TempText2 = Trim(Mid(TempText1, 1, ChrPointer - 1))
        ValSec = Val(TempText2)

        '- Extracting value Sign (+/-, ValSgn)
        TempText2 = Mid(TempText1, ChrPointer + 1, Len(TempText1) - ChrPointer)
        Select Case TempText2
            Case "W", "S"
                ValSign = -1
            Case "N", "E"
                ValSign = 1
        End Select

        '- Creating return value of function and ending
        Return Format(((ValDeg + (ValMin / 60) + (ValSec / 3600)) * ValSign), "0.000000")

    End Function

    'The Function PixelSize calculates the pixel size based on the pixel and grid coordinates of the image corners
    '-  The size of pixel is in units of the geographical coordinate system
    '-  Also used to construct a boundary based on the the NEATLINE.
    Public Function PixelSize(TopLeft As Coordinate, TopRight As Coordinate, BottomRight As Coordinate, BottomLeft As Coordinate) As Double
        '- Declarations
        Dim Delta(4) As Double              'Function internal array; represents distance (delta) between two points
        '   1 = topleft to topright
        '   2 = topright tp bottomright
        '   3 = bottomright to bottomleft
        '   4 = bottomleft to topleft
        '   0 = topleft to bottomright
        Dim PreSize(4) As Double            'Function internal array; represents the dotsize for delta 0 to 4
        Dim AvgSize As Double               'Function internal varable; reprsents the avarage dotsize (= return value for function)

        Dim PixelWidth As Double            'Width of image in pixel
        Dim PixelHeight As Double           'Height of image in pixel
        Dim PixelDiagonal As Double         'Length of diagonal of image in pixel

        PixelWidth = Val(BottomRight.x)
        PixelHeight = Val(BottomRight.y)
        PixelDiagonal = (PixelWidth ^ 2 + PixelHeight ^ 2) ^ 0.5

        '- Calculating Delta's (distance between corners coordinates
        Delta(1) = ((Val(TopLeft.east) - Val(TopRight.east)) ^ 2 + (Val(TopLeft.north) - Val(TopRight.north)) ^ 2) ^ 0.5
        Delta(2) = ((Val(TopRight.east) - Val(BottomRight.east)) ^ 2 + (Val(TopRight.north) - Val(BottomRight.north)) ^ 2) ^ 0.5
        Delta(3) = ((Val(BottomRight.east) - Val(BottomLeft.east)) ^ 2 + (Val(BottomRight.north) - Val(BottomLeft.north)) ^ 2) ^ 0.5
        Delta(4) = ((Val(BottomLeft.east) - Val(TopLeft.east)) ^ 2 + (Val(BottomLeft.north) - Val(TopLeft.north)) ^ 2) ^ 0.5
        Delta(0) = ((Val(TopLeft.east) - Val(BottomRight.east)) ^ 2 + (Val(TopLeft.north) - Val(BottomRight.north)) ^ 2) ^ 0.5

        '- Calculating "individual" DotSize
        PreSize(1) = Delta(1) / PixelWidth
        PreSize(2) = Delta(2) / PixelHeight
        PreSize(3) = Delta(3) / PixelWidth
        PreSize(4) = Delta(4) / PixelHeight
        PreSize(0) = Delta(0) / PixelDiagonal   'special case: testing grid is square (PreSize must ~ AvgRatio)

        '- Calculating average DotSize
        AvgSize = (PreSize(1) + PreSize(2) + PreSize(3) + PreSize(4)) / 4

        '- Scanning the difference between PreRatio and AvgRatio > to much > No boundary (AvgSize will set to 0)
        If Math.Abs((PreSize(1) - AvgSize) / AvgSize) > 0.005 Or
                Math.Abs((PreSize(2) - AvgSize) / AvgSize) > 0.005 Or
                Math.Abs((PreSize(3) - AvgSize) / AvgSize) > 0.005 Or
                Math.Abs((PreSize(4) - AvgSize) / AvgSize) > 0.005 Or
                Math.Abs((PreSize(0) - AvgSize) / AvgSize) > 0.005 Then
            AvgSize = 0
        End If

        '- Returning value of function
        Return AvgSize

    End Function

    'Function ConvertToMaxLineSize checks length of line in the SourceFile and split it, if needed in smaller lines 
    Public Function ConvertToMaxLineSize(SourceString As String, MaxNumberOfCharacters As Integer) As String
        'Declarations
        Dim TargetString As String = ""             'variable used to build the result (string)
        Dim TargetLine As String                    'Variable represents a string with a max size of MaxNumberOfCharacters
        Dim TempLine As String = ""                 'Variable used to break up sourcelines
        Dim CharacterPointer As Integer             'Variable used to jump from character to character in a string

        '- Spliting SourceString into lines
        Dim SourceLine() As String = Split(SourceString, EndLine, )     'Array with all lines in SourceString
        Dim LastSourceLine As Integer = SourceLine.Count - 1            'Variable represents last record number of SourceLine array
        Dim SourceLinePointer As Integer = -1                           'Variabel used as pointer in relation to SourceLine

        '- Line by line checking a Sourceline
        Do Until SourceLinePointer = LastSourceLine And TempLine = ""

            '- Reading next SourceLine
            If TempLine = "" Then
                SourceLinePointer = SourceLinePointer + 1
                TempLine = SourceLine(SourceLinePointer)
            End If

            '- Handling a TempLine, Checking length of string and, if needed break it in to smaller lines. 
            If Len(TempLine) > MaxNumberOfCharacters Then
                TargetLine = Strings.Left(TempLine, MaxNumberOfCharacters)
                '- If possible breaking a line on space or a ,
                '  else breaking at MaxNumberOfCharacters

                If Strings.InStr(TargetLine, " ") > 0 Or Strings.InStr(TargetLine, ",") > 0 Then
                    For CharacterPointer = Len(TargetLine) To 0 Step -1

                        '-  "walk back" and testing for space or '
                        If Strings.Mid(TargetLine, CharacterPointer, 1) = " " Or Strings.Mid(TargetLine, CharacterPointer, 1) = "," Then
                            Exit For
                        End If

                    Next

                    TargetLine = Strings.Left(TargetLine, CharacterPointer)

                Else
                    TargetLine = Strings.Left(TempLine, MaxNumberOfCharacters)

                End If
                '- resizing TempLine for next loop
                TempLine = Strings.Right(TempLine, Len(TempLine) - Len(TargetLine))

            Else
                TargetLine = TempLine
                TempLine = ""

            End If

            '- Writing TargetLine to TargetString
            TargetString = TargetString & TargetLine & EndLine

        Loop

        Return TargetString

    End Function

    'Subroutine BuildPages builds the array PageToPrint with MaxCharsPerLine and MaxLinesPerPage
    Public Sub BuildPages(SourceString As String, MaxCharsPerLine As Integer, MaxLinesPerPage As Integer)
        '- Resizing the lines in the SourceString using function ConvertToMaxLineSize
        SourceString = ConvertToMaxLineSize(SourceString, MaxCharsPerLine)

        '- Spliting SourFile into lines
        Dim TempLine() As String = Split(SourceString, EndLine)
        NumberOfLines = TempLine.Count - 1
        NumberOfPages = NumberOfLines \ MaxLinesPerPage

        ReDim PageToPrint(NumberOfPages)

        'Building PageToPrint line by line
        For LinePointer = 0 To NumberOfLines
            PagePointer = LinePointer \ MaxLinesPerPage
            If LinePointer Mod MaxLinesPerPage = 0 Then
                PageToPrint(PagePointer) = TempLine(LinePointer) & EndLine
            Else
                PageToPrint(PagePointer) = PageToPrint(PagePointer) & TempLine(LinePointer) & EndLine
            End If

        Next

    End Sub

    '- Function CreateImageData returns height, width, horizontal and vertical resolution, colordepth (as string)
    '  and an indication if the data could extracted
    Public Function CreateImageData(ImageFileName As String) As ImageDataStructure
        '- Declarations
        Dim TempImageData As ImageDataStructure

        '- Setting values in case image file doesen't exits
        If My.Computer.FileSystem.FileExists(ImageFileName) = False Then
            TempImageData.Exists = False        '(false means file couldnit found)
            TempImageData.Valid = False         '(false means data Couldn't loaded)
            TempImageData.Usable = False        '(False means the colordepth can't be used by M-M)
            TempImageData.PixelSizeX = 0
            TempImageData.PixelSizeY = 0
            TempImageData.ResolutionX = 0
            TempImageData.ResolutionY = 0
            TempImageData.ColorDepth = ""

            '- Loading the return values
            Return TempImageData

            Exit Function

        End If

        '- Initiating error handling (If image is to large the Error kicks in)
        On Error GoTo Errorhandler

        '- Loading image (if not loaded > to errorhandler)
        TempImage = Image.FromFile(ImageFileName)

        TempImageData.Exists = True         '(true means file was found)
        TempImageData.Valid = True          '(true means data Couldn't loaded)

        '- Extracting data from image
        TempImageData.PixelSizeX = TempImage.Width
        TempImageData.PixelSizeY = TempImage.Height
        TempImageData.ResolutionX = TempImage.HorizontalResolution
        TempImageData.ResolutionY = TempImage.VerticalResolution
        TempImageData.ColorDepth = TempImage.PixelFormat.ToString

        '- Handling colordepth related to M-M
        Select Case TempImageData.ColorDepth
            Case "Format8bppIndexed", "Format4bppIndexed", "Format1bppIndexed"      'The 'acceptable' colordepths 
                TempImageData.Usable = True

            Case Else
                TempImageData.Usable = False

        End Select

        '- Loadding the return values
        Return TempImageData

        Exit Function

Errorhandler:
        '- Setting values to "zero"
        TempImageData.Exists = True         '(true means file was found)
        TempImageData.Valid = False         '(false means data Couldn't loaded)
        TempImageData.Usable = False        '(False means the colordepth can't be used by M-M)
        TempImageData.PixelSizeX = 0
        TempImageData.PixelSizeY = 0
        TempImageData.ResolutionX = 0
        TempImageData.ResolutionY = 0
        TempImageData.ColorDepth = ""

        '- loading the return values
        Return TempImageData

        Exit Function

    End Function

    'Function UsedGdal returns the used gdal version
    Public Function UsedGdal() As String
        Dim Result As String        'variable used to build the result
        Dim TempFile As String      'variable used to represents a temporary file (to store GDAL version)
        Dim CommandLine As String   'variable used to build the commandline (to be used in the shell command)

        '- Retrieving teh TEMP directory
        TempFile = My.Computer.FileSystem.SpecialDirectories.Temp & "\" & "m4mm.tmp"

        '- Building the CommandLine
        CommandLine = C34 & BatFile.LongName & C34 & " gdalinfo.exe --version > " & C34 & TempFile & C34

        '- Executing CommandLine and saving the result in m4.mm.tmp in the TEMP directory
        Call Shell(CommandLine, AppWinStyle.Hide, Wait:=True)

        '- Extracting the text from m4mm.tmp 
        Result = My.Computer.FileSystem.ReadAllText(TempFile)
        Result = Strings.Left(Result, Len(Result) - 2)   ' Cutting last two (endline) characters 

        '  - Deleting TempDataFileName m4mm.tmp
        My.Computer.FileSystem.DeleteFile(TempFile)

        Return Result

    End Function

    'Function WktText returns the WKT 2.0 text included GeoTIFF, GeoPDF and MrSID image files
    Public Function WktText(ImageFile As String) As String
        Dim Result As String            'variable used to build the result of the function
        Dim TempFile As String          'variable used to represents a temporary file (to store WKT)
        Dim CommandLine As String       'variable used to build the commandline (to be used in the shell command)

        '- Creating name of TempFile (in the TEMP directory)
        TempFile = My.Computer.FileSystem.SpecialDirectories.Temp & "\" & "m4mm.tmp"

        '- Buiding the content for the commandline
        CommandLine = C34 & BatFile.LongName & C34 & " gdalinfo.exe -wkt_format WKT2 " & C34 & ImageFile & C34 & " > " & C34 & TempFile & C34

        '- Executing commandline
        Call Shell(CommandLine, AppWinStyle.Hide, Wait:=True)

        '- Retrieving text from TempFile 
        Result = My.Computer.FileSystem.ReadAllText(TempFile)

        '- Deleting TempDataFileName m4mm.tmp
        My.Computer.FileSystem.DeleteFile(TempFile)

        Return Result

    End Function

End Module
