﻿Imports System.ComponentModel

Public Class ImageInfo
    'Declarations
    Public MapImage As Image                       'Object; creates an image object for extracting data from image file

    'Subroutine ImageInfo_Load
    '- Actions at opening the form ImageInfo
    Private Sub ImageInfo_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        '- Declarations
        Dim TipText As New ToolTip()                'Creating tooltips for this form

        '- Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        '- Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        '- Tooltips
        TipText.SetToolTip(TopStrip, "Select a image file to view the content. If a file is selected the image can be viewed.")
        TipText.SetToolTip(ImageSizeValue, "Image size in pixel (dots).")
        TipText.SetToolTip(ResolutionValue, "Resolution in DPI.")
        TipText.SetToolTip(ColorDepthValue, "Description of teh colordepth.")
        TipText.SetToolTip(FileSizeValue, "Size of selected file in byte.")

        '- Disabling StartMemu form
        StartMenu.Enabled = False

        '- Setting form label
        Me.Text = AppName & " - Image information"

        '- Initiating SelectItem
        SelectItem_Click(sender, e)

    End Sub
    'Subroutine ImageInfo_Closing
    '- Actions at closing the form ImageInfo
    Private Sub ImageInfo_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing

        '- Enabling StartMenu form
        StartMenu.Enabled = True

    End Sub

    'Subroutine CloseItem_Click
    '- initializing closing the form ImageInfo
    Private Sub CloseItem_Click(sender As Object, e As EventArgs) Handles CloseItem.Click
        Me.Close()

    End Sub

    'Subroutine SelectButton_Click
    '- Selecting TIF or PNG file
    '- Extractig image data form image file
    '- Loading image, data and filename into the form
    Private Sub SelectItem_Click(sender As Object, e As EventArgs) Handles SelectItem.Click
        '- Declarations
        Dim SelectFile As New OpenFileDialog()      'Creating an File dialog object 

        '- Clearing all values in form
        FileNameValue.Text = ""
        ImageSizeValue.Text = ""
        ResolutionValue.Text = ""
        ColorDepthValue.Text = ""
        FileSizeValue.Text = ""

        '- Loading image (NOfile) form resources
        ImageBox.Image = M4MM.My.Resources.nofile

        '- Setting patameters for SelectFile
        With SelectFile
            .Filter = "Graphics (*.png,*.tif)|*.png; *.tif; *.tiff|PNG-Files (*.png)|*.png|TIFF-files (*.tif)|*.tif; *.tiff"
            .Multiselect = False
        End With

        '- Triggering SelectFile and handling case no file was selected
        If SelectFile.ShowDialog <> DialogResult.OK Then

            '- Warning
            MsgBox("No file was selected!", 48, AppName & " - Warning")

            '- Loading image (nofile) form resources
            ImageBox.Image = M4MM.My.Resources.nofile

            Exit Sub

        End If

        '- Redirection to ErrorHandler in case image size is to large
        '  (MapImage = Image.FromFile(SelectFile.FileName))
        On Error GoTo ErrorHandler

        '- Opening image
        MapImage = Image.FromFile(SelectFile.FileName)      'opening image

        '- Showing selected filename in form (FileValue)
        FileNameValue.Text = SelectFile.FileName

        '- Getting file size (in byte) and colorizing it
        FileSizeValue.Text = FileSystem.FileLen(SelectFile.FileName)
        FileSizeValue.ForeColor = Color.Black

        '- Getting image size (in pixel)
        ImageSizeValue.Text = MapImage.Width & ", " & MapImage.Height
        ImageSizeValue.ForeColor = Color.Black

        '- Getting image resolution in DPI 
        ResolutionValue.Text = MapImage.HorizontalResolution & ", " & MapImage.VerticalResolution

        '- Getting colordepth in text
        ColorDepthValue.Text = MapImage.PixelFormat.ToString

        '- Colorizing ColorDepthValue (in case not usable)
        Select Case ColorDepthValue.Text
            Case "Format8bppIndexed", "Format4bppIndexed", "Format1bppIndexed"      'The 'acceptable' colordepths 
                ColorDepthValue.ForeColor = Color.Black
            Case Else
                ColorDepthValue.ForeColor = Color.Red
                MsgBox("The image doesn't have the propper colordepth to use with Memory-Map!", 48, AppName & " - Warning")

        End Select

        '- Displaying MapImage
        ImageBox.Image = MapImage

        Exit Sub

ErrorHandler:

        '-  Warning 
        MsgBox("The file " & SelectFile.FileName & " is too large to process!", 48, AppName & " - Warning")

        '- Showing selected filename in form (FileValue)
        FileNameValue.Text = SelectFile.FileName

        '- Loading image (largefile) form resources
        ImageBox.Image = M4MM.My.Resources.largefile

        '- Getting file size (in byte) and colorizing it
        FileSizeValue.Text = FileSystem.FileLen(SelectFile.FileName)
        FileSizeValue.ForeColor = Color.Red

        Exit Sub

    End Sub

    Private Sub ImageInfo_Resize(sender As Object, e As EventArgs) Handles Me.Resize

        '- Resizing ImageBox to 'maximum' height/width within the form
        ImageBox.Width = ClientSize.Width - 200
        ImageBox.Height = ClientSize.Height - 100

    End Sub

End Class