﻿Imports System.ComponentModel

Public Class JprEdit

    '- Variable declarations
    Public JprFile As FileNameStructure             'Variable; represents the JPR-filename
    Public BakFile As FileNameStructure             'Variable; represents the filename of the JPR-backup file
    Public StartText As String                      'Variable; represents the orignal content of JPR-file (used to compare for changes in JPR-Data)

    'Subroutine JprEdit_Load
    '- Actions at opening the form JprEdit
    Private Sub JprEdit_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        '- Declarations
        Dim TipText As New ToolTip()                'Creating tooltips for this form

        '- Clearing all values in form
        FileValue.Text = ""
        DataBox.Text = ""

        '- Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        '- Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        '- Tooltips
        TipText.SetToolTip(TopStrip, "Select a JPR-file to edit the content or save the JPR-file below.")
        TipText.SetToolTip(DataBox, "Content of JPR-file.")
        TipText.SetToolTip(JprInfoBox, "Explanation of the variables used in a JPR-file.")

        '- Loading jprinfo.txt in tyo JprInfoBox
        '  (jprinfo.txt is defined as a resource (file))
        JprInfoBox.Text = My.Resources.jprinfo

        '- Disabling objects (until they can be used)
        SaveItem.Enabled = False
        PrintItem.Enabled = False
        DiscardItem.Enabled = False
        DataBox.Enabled = False

        '- Setting form title
        Me.Text = AppName & " - JPR-file Edit"

        '- Disabling StartMenu form
        StartMenu.Enabled = False

        '- Moving cursor to DataBox and than to SelectItem
        '  (preventing highlighting text in InfoBox)
        DataBox.Select()
        SelectItem.Select()

        '- Initiating SelectItem
        SelectItem_Click(sender, e)

    End Sub

    'Subroutine CloseItem_Click
    '- Checking and if needed, save edited JPR-data 
    '- initializing closing the form JprEdit
    Private Sub CloseItem_Click(sender As Object, e As EventArgs) Handles CloseItem.Click
        '- Decalrations
        Dim Answer As Integer       'Subroutine internal variable; represents answer value for MsgBox

        '- Comparing DataBox.Text with StartText to prevent unwished deleting 
        If DataBox.Text <> StartText Then
            '(Msgbox appearance: 4 = Yes/No, 32 = Question mark, 36 = total)
            Answer = MsgBox("The JPR-data aren't saved. Do you want to save them?", 36, "Warning")
            If Answer = 6 Then          '6 = yes
                SaveItem_Click(sender, e)       'calling save
            Else
                DiscardItem_Click(sender, e)    'calling discard
            End If

        End If

        Me.Close()

    End Sub

    'Subroutine JprEdit_Closing
    '- Actions at closing the form JprEdit
    Private Sub JprEdit_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        '- Enabling M4MM form
        StartMenu.Enabled = True

    End Sub

    'Subroutine SelectItem_Click
    '- Selecting JPR-file
    '- Loading data and filename into the form
    Private Sub SelectItem_Click(sender As Object, e As EventArgs) Handles SelectItem.Click

        '- Clearing all values in form
        FileValue.Text = ""
        DataBox.Text = ""
        StartText = ""

        '- Setting patameters for SelectFile
        With SelectFile
            .Filter = "JPR-file (*.jpr)|*.jpr"
            .Multiselect = False
        End With

        '- Triggering SelectFile and handling case no file was selected
        If SelectFile.ShowDialog <> DialogResult.OK Then

            FileValue.Text = ""
            MsgBox("No file was selected!", 48, AppName & " - Warning")

            '- Disabling Print menu items
            SaveItem.Enabled = False
            PrintItem.Enabled = False
            DiscardItem.Enabled = False
            DataBox.Enabled = False

            Exit Sub

        End If

        '- Setting values for Filenames
        JprFile = SplitFileName(SelectFile.FileName)
        BakFile = SplitFileName(JprFile.Path & JprFile.ShortName & ".bak")

        '- Showing selected filename in form (FileValue)
        FileValue.Text = SelectFile.FileName

        '- Writing JPR-data to form
        DataBox.Text = My.Computer.FileSystem.ReadAllText(SelectFile.FileName)

        '- Writing Jpr-data to StartText 
        StartText = DataBox.Text

        '- Enabling objects (until they can be used)
        SaveItem.Enabled = True
        PrintItem.Enabled = True
        DiscardItem.Enabled = True
        DataBox.Enabled = True

    End Sub

    'Subroutine SaveItem_Click
    '- creating a backup file of the orignal JPR data
    '- Saving edited data to a new JPR-file
    Private Sub SaveItem_Click(sender As Object, e As EventArgs) Handles SaveItem.Click

        '- Creating a backup of the JPR-file
        FileCopy(JprFile.LongName, BakFile.LongName)

        '- Writing data from DataBox to JPR-file
        My.Computer.FileSystem.WriteAllText(JprFile.LongName, DataBox.Text, False)

        '- Resetting StartText
        StartText = DataBox.Text

    End Sub

    'Subroutine DiscardItem_Click
    '- Clearing content of DataBox
    Private Sub DiscardItem_Click(sender As Object, e As EventArgs) Handles DiscardItem.Click

        '- Clearing DataBox and StartText
        DataBox.Text = ""
        StartText = ""

        '- Disabling objects(until they can be used)
        SaveItem.Enabled = False
        DiscardItem.Enabled = False
        DataBox.Enabled = False


    End Sub

    'Subroutine RetrieveItem_Click
    '- Selecting BAK-file
    '- Loading data and filename into the form
    Private Sub RetrieveItem_Click(sender As Object, e As EventArgs) Handles RetrieveItem.Click
        '- Declarations
        Dim SelectFile As New OpenFileDialog()      'Creating an File dialog object

        '- Clearing all values in form
        FileValue.Text = ""
        DataBox.Text = ""
        StartText = ""

        '- Setting up dialog
        SelectFile.Filter = "backup-file (*.bak)|*.bak"
        SelectFile.Multiselect = False

        '- Opening Dialog
        SelectFile.ShowDialog()

        '- Handling case no file was selected
        If Len(SelectFile.FileName) = 0 Then
            'ViewItem.Enabled = False
            FileValue.Text = ""
            MsgBox("No file was selected!", 48, AppName & " - Warning")

            '- Disabling objects (until they can be used)
            SaveItem.Enabled = False
            DiscardItem.Enabled = False
            DataBox.Enabled = False

            Exit Sub
        End If

        '- Setting values for Filenames
        BakFile = SplitFileName(SelectFile.FileName)
        JprFile = SplitFileName(JprFile.Path & JprFile.ShortName & ".jpr")

        '- Showing selected filename in form (FileValue)
        FileValue.Text = SelectFile.FileName

        '- Writing JPR-data to form
        DataBox.Text = My.Computer.FileSystem.ReadAllText(SelectFile.FileName)

        '- Writing Jpr-data to StartText 
        StartText = DataBox.Text

        '- Enabling objects (until they can be used)
        SaveItem.Enabled = True
        DiscardItem.Enabled = True
        DataBox.Enabled = True

    End Sub

    'Inintializing the printing of the content of teh DataBox 
    Private Sub PrintDataItem_Click(sender As Object, e As EventArgs) Handles PrintDataItem.Click

        StartMenu.PrintContent(DataBox.Text, DataBox.Font, LandscapeItem.Checked)

    End Sub

End Class