﻿Imports System.ComponentModel

Public Class OziBuild

    '- Declarations
    Public MapFile As FileNameStructure             'Variable; represents filename MAP-file.
    Public JprFile As FileNameStructure             'Variable; represents filename JPR-file.
    Public ImageFile As FileNameStructure           'Variable; represents filename of file containing image of map (graphical or ozf)
    Public GeoData As String                        'Variable; represents the content of the MAP-file
    Public GeoDataLen As Integer                    'Variable; represents (text) length of GeoData
    Public GeoDataLine(60) As String                'array; represents content of each line in MapFile content (will be (re)dimensioned for every MAP-file)
    Public GeoDataLineCount As Integer              'Variable; represents amount Of lines In GeoData
    Public GeoDataLineField(20) As String           'Array; represents content of a field in a GeoDataLine (used to extract data)
    Public JprData As JprStructure                  'Variable; represents the JPR data
    Public ReferencePoint(3) As Coordinate          'Array; represents reference points (will be (re)dimensioned for every MAP-file)
    Public ReferencePointCounter As Integer         'Variable; represents the counted amount of reference points in MAP-file
    Public ValidReferencePoints As Integer          'Variable; represents the amount of usable Number of reference points in MAP-file
    Public VertexPoint(4) As Coordinate             'Array; represents boundary points (will be (re)dimensioned for every MAP-file)
    Public VertexPointNoted As Integer              'Variable; represents the amount of boundary points number of boundary points in MAP-file 
    Public ValidVertexPoints As Integer             'Variable; represents the amount of usable boundary points in MAP-file 
    Public VertexMax As Integer                     'Variable; represents the amount maximum boundary points in MAP-file
    Public VertexPointLLCounter As Integer          'Variable; represents the amount of boundary points based on Latitude/longitude
    Public VertexPointXYCounter As Integer          'Variable; represents the amount of boundary points based on X/Y coordinate

    'Subroutine OziBuild_Load
    '- Actions at opening the form OziBuild
    Private Sub OziBuild_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        '- Declarations
        Dim TipText As New ToolTip()                'Creating tooltips for this form

        '- Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        '- Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        '- Tooltips
        TipText.SetToolTip(TopStrip, "The workflow: 1. Select a file, 2. extract the data, 3. build the JPR-file and 4. Save the JPR-file.")
        TipText.SetToolTip(GeoDataBox, "Content of Ozi MAP-file.")
        TipText.SetToolTip(ExtractDataBox, "Comment on and extracted data from Ozi MAP-file.")
        TipText.SetToolTip(GeoDataBox, "Content for JPR-file.")

        '- Clearing all values in form
        FileValue.Text = ""
        GeoDataBox.Text = ""
        ExtractDataBox.Text = ""
        JprDataBox.Text = ""

        '- Adding values to DpiBox
        DpiBox.Items.Add("72")
        DpiBox.Items.Add("96")
        DpiBox.Items.Add("100")
        DpiBox.Items.Add("127")
        DpiBox.Items.Add("150")
        DpiBox.Items.Add("200")
        DpiBox.Items.Add("254")
        DpiBox.Items.Add("300")
        DpiBox.Items.Add("508")
        'Default setting
        DpiBox.Text = "254"

        '- Adding values to ScaleBox
        ScaleBox.Items.Add("12500")
        ScaleBox.Items.Add("20000")
        ScaleBox.Items.Add("25000")
        ScaleBox.Items.Add("30000")
        ScaleBox.Items.Add("33333")
        ScaleBox.Items.Add("40000")
        ScaleBox.Items.Add("50000")
        ScaleBox.Items.Add("66666")
        ScaleBox.Items.Add("100000")
        ScaleBox.Items.Add("200000")
        ScaleBox.Items.Add("250000")
        ScaleBox.Items.Add("300000")
        ScaleBox.Items.Add("500000")
        'Default setting
        ScaleBox.Text = "50000"


        '- Disabling StartMenu form
        StartMenu.Enabled = False

        '- Setting form title
        Me.Text = AppName & " - Build JPR-file from Ozi MAP-file"

        '- Disabling TopStrip items (until item can be used)
        ExtractItem.Enabled = False
        BuildItem.Enabled = False
        SaveItem.Enabled = False

        '- Disabling groups
        SelectGroup.Enabled = False
        ExtractGroup.Enabled = False
        JprGroup.Enabled = False

        '- Disabling Print menu items
        PrintItem.Enabled = False

        '- Initiating SelectItem
        SelectItem_Click(sender, e)

    End Sub

    'Subroutine CloseItem_Click()
    '- initializing closing the form OziBuild
    Private Sub CloseItem_Click(sender As Object, e As EventArgs) Handles CloseItem.Click
        Me.Close()

    End Sub
    'Subroutine OziBuild_Closing
    '- Actions at closing the form OziBuild
    Private Sub OziBuild_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        '- Enabling StartmMenu form
        StartMenu.Enabled = True

    End Sub

    'Subroutine SelectItem_Click
    '- Selecting Ozi MAP-file
    '- Extractig data from Ozi MAP-file
    '- Writing data and filename to the form
    '- Creating values for variable MapFile
    Private Sub SelectItem_Click(sender As Object, e As EventArgs) Handles SelectItem.Click
        '- Declarations
        Dim SelectFile As New OpenFileDialog()      'Creating an File dialog object 

        '- Clearing all values in form and disabling groups and TopStrip items
        FileValue.Text = ""
        GeoDataBox.Text = ""
        ExtractDataBox.Text = ""
        JprDataBox.Text = ""
        SelectGroup.Enabled = True
        ExtractGroup.Enabled = False
        JprGroup.Enabled = False
        ExtractItem.Enabled = False
        BuildItem.Enabled = False
        SaveItem.Enabled = False

        '- Clearing and resetting  all variables
        ImageWidth = 0
        ImageHeight = 0
        ImageDiagonal = 0
        GeoData = ""
        ClearJprData()

        '- Setting patameters for SelectFile
        With SelectFile
            .Filter = "Ozi MAP-file(*.map)|*.map"
            .Multiselect = False
        End With

        '- Triggering SelectFile and handling case no file was selected
        If SelectFile.ShowDialog <> DialogResult.OK Then

            FileValue.Text = ""
            MsgBox("No file was selected!", 48, AppName & " - Warning")

            '- Disabling menu items
            ExtractItem.Enabled = False
            BuildItem.Enabled = False
            SaveItem.Enabled = False
            FileValue.Text = ""
            PrintItem.Enabled = False

            Exit Sub

        End If

        '- Setting Sourcefile
        MapFile = SplitFileName(SelectFile.FileName)

        '- Showing selected filename in form (FileValue)
        FileValue.Text = MapFile.LongName

        '- Setting Value for GeoData and writing these data to form
        GeoData = My.Computer.FileSystem.ReadAllText(SelectFile.FileName)
        GeoDataBox.Text = GeoData

        '- Enabling next steps in the application and moving text cursor to GeoDataBox
        SelectGroup.Enabled = True
        GeoDataBox.Select()
        GeoDataBox.DeselectAll()
        ExtractItem.Enabled = True
        ExtractItem.Select()

        '- Enabling Print menu items
        PrintItem.Enabled = True

    End Sub

    'Subroutine ExtractItem_Click
    '- Checking if the MAP-file is a OZI-file
    '- Creating values for variable JprFile
    '- Breaking content of Ozi MAP-file in to lines and load them in an array (LineContent)
    '- Retrieving all neccesary JPR-data
    Private Sub ExtractItem_Click(sender As Object, e As EventArgs) Handles ExtractItem.Click
        'Declarations
        Dim LinePointer As Integer                                      'Subroutine internal variable; points to a specific line
        Dim TempText As String                                          'Subroutine internal variable; represents a string
        Dim Pointer As Integer                                          'Subroutine internal variable; used as an array pointer

        'Checking if MAP-data contain OziExplorer information
        If InStr(GeoData, "OziExplorer") = 0 Then
            ExtractDataBox.Text = ExtractDataBox.Text & "The file " & MapFile.LongName & " wasn't created with or for OziExplorer."
            ExtractItem.Enabled = False
            Exit Sub
        End If

        '- Clearing all values in form and disabling groups and TopStrip items
        ExtractDataBox.Text = ""
        JprDataBox.Text = ""
        JprGroup.Enabled = False
        BuildItem.Enabled = False
        SaveItem.Enabled = False

        '- Clearing JprData
        ClearJprData()

        '- Spliting GeoData into GeoDataLine Array
        Dim TempLine() As String = Split(GeoData, EndLine)
        GeoDataLineCount = TempLine.Length - 1           'Determening number of lines

        ReDim GeoDataLine(GeoDataLineCount)

        For LinePointer = 0 To GeoDataLineCount - 1
            GeoDataLine(LinePointer + 1) = TempLine(LinePointer)
        Next

        '- Retrieving and setting value JprData.nm
        JprData.nm = LCase(GeoDataLine(2))
        ExtractDataBox.Text = ExtractDataBox.Text & "nm (map name) = " & JprData.nm & EndLine

        '- Retrieving and setting value JprData.it
        ImageFile = SplitFileName(GeoDataLine(3))
        TempText = LCase(ImageFile.Ext)

        If My.Computer.FileSystem.FileExists(ImageFile.LongName) = True Then        'Image file is located as mentioned in MAP-file
            JprFile = SplitFileName(ImageFile.Path & ImageFile.ShortName & ".jpr")  'JPR-file will be generated on the same location

        Else                                                                        'Image file isn't located as mentioned in MAP-file
            JprFile = SplitFileName(MapFile.Path & ImageFile.ShortName & ".jpr")    'JPR-file will be generated on location MAP-file

        End If

        '- Evaluating Image type (it)
        Select Case TempText
            Case "tif", "tiff"
                JprData.it = "tif"
                ExtractDataBox.Text = ExtractDataBox.Text & "it (image type) = " & JprData.it & ". This image map must be convereted to png." & EndLine
            Case "png"
                JprData.it = "png"
                ExtractDataBox.Text = ExtractDataBox.Text & "it (image type) = " & JprData.it & EndLine
            Case "ozf2", "ozfx3"
                JprData.it = TempText
                ExtractDataBox.Text = ExtractDataBox.Text & "it (image type) = " & JprData.it & ". This OziExplorer map must be convereted to png." & EndLine
            Case "gif", "bmp", "jpg"
                JprData.it = TempText
                ExtractDataBox.Text = ExtractDataBox.Text & "it (image type) = " & JprData.it & ". This image file must be convereted to png." & EndLine
            Case Else
                JprData.it = TempText
                ExtractDataBox.Text = ExtractDataBox.Text & "it (image type) = " & JprData.it & ". This image type isn't supported." & EndLine
        End Select

        '- Retrieving data for the boundary

        '- Resetting boundary variables
        VertexPointNoted = 0
        VertexPointLLCounter = 0
        VertexPointXYCounter = 0
        ValidVertexPoints = 0

        '- Retrieving amount of boundary points
        For LinePointer = 41 To GeoDataLineCount
            SplitLine(GeoDataLine(LinePointer))
            If LCase(GeoDataLineField(1)) = "mmpnum" Then
                VertexPointNoted = Val(GeoDataLineField(2))
            End If

        Next

        If VertexPointNoted < 3 Then
            ExtractDataBox.Text = ExtractDataBox.Text & "Number of boundary points is to less to construct a boundary" & EndLine
        Else
            '- Resetting VertexPoint array
            ReDim VertexPoint(VertexPointNoted)
            For Pointer = 1 To VertexPointNoted
                VertexPoint(Pointer) = ClearCoordinate()
            Next

            '- Retrieving the data for boundary points
            For LinePointer = 41 To GeoDataLineCount
                Call SplitLine(GeoDataLine(LinePointer))

                If LCase(GeoDataLineField(1)) = "mmpll" And Val(GeoDataLineField(2)) <= VertexPointNoted Then
                    If Trim(GeoDataLineField(3)) = "" Or Trim(GeoDataLineField(4)) = "" Then
                        VertexPoint(Val(GeoDataLineField(2))).Valid = VertexPoint(Val(GeoDataLineField(2))).Valid + 1
                    Else
                        VertexPoint(Val(GeoDataLineField(2))).lon = Val(GeoDataLineField(3))
                        VertexPoint(Val(GeoDataLineField(2))).lat = Val(GeoDataLineField(4))
                    End If

                End If

                If LCase(GeoDataLineField(1)) = "mmpxy" And Val(GeoDataLineField(2)) <= VertexPointNoted Then
                    VertexPointXYCounter = VertexPointXYCounter + 1
                    If Trim(GeoDataLineField(3)) = "" Or Trim(GeoDataLineField(4)) = "" Then
                        VertexPoint(Val(GeoDataLineField(2))).Valid = VertexPoint(Val(GeoDataLineField(2))).Valid + 2
                    Else
                        VertexPoint(Val(GeoDataLineField(2))).x = Val(GeoDataLineField(3))
                        VertexPoint(Val(GeoDataLineField(2))).y = Val(GeoDataLineField(4))
                    End If

                End If
            Next

            '- Validation of amount of boundary points
            For Pointer = 1 To VertexPointNoted
                If VertexPoint(Pointer).Valid = 0 Then
                    ValidVertexPoints = ValidVertexPoints + 1
                End If
            Next

            If VertexPointNoted <> ValidVertexPoints Then
                ExtractDataBox.Text = ExtractDataBox.Text & ValidReferencePoints & "Amount of boundary points is inconclusive! No boundary can be constructed." & EndLine
            Else
                ExtractDataBox.Text = ExtractDataBox.Text & "A boundary with " & VertexPointNoted & " points can be constructed." & EndLine
            End If

        End If

        '- Retrieving data for Reference points

        '- Determening number of reference points (RefPoint array)
        ReferencePointCounter = 0
        For LinePointer = 10 To 39
            Call SplitLine(GeoDataLine(LinePointer))
            If Trim(GeoDataLineField(3)) <> "" Then
                ReferencePointCounter = ReferencePointCounter + 1
            Else
                Exit For
            End If
        Next

        '- Resetting ReferencePoint Array
        ReDim ReferencePoint(ReferencePointCounter)
        For Pointer = 1 To ReferencePointCounter
            ReferencePoint(Pointer) = ClearCoordinate()
        Next

        '- Determening values for the Reference points array and number of valid Reference points
        ValidReferencePoints = 0
        For LinePointer = 10 To 9 + ReferencePointCounter
            Call SplitLine(GeoDataLine(LinePointer))
            If Trim(GeoDataLineField(7)) <> "" And Trim(GeoDataLineField(10)) <> "" Then
                ReferencePoint(LinePointer - 9).x = Val(GeoDataLineField(3))
                ReferencePoint(LinePointer - 9).y = Val(GeoDataLineField(4))
                ReferencePoint(LinePointer - 9).lat = Val(GeoDataLineField(7)) + Val(GeoDataLineField(8)) / 60
                If Trim(LCase(GeoDataLineField(9))) = "s" Then ReferencePoint(LinePointer - 9).lat = ReferencePoint(LinePointer - 9).lat * -1
                ReferencePoint(LinePointer - 9).lon = Val(GeoDataLineField(10)) + Val(GeoDataLineField(11)) / 60
                If Trim(LCase(GeoDataLineField(12))) = "w" Then ReferencePoint(LinePointer - 9).lon = ReferencePoint(LinePointer - 9).lon * -1
                ValidReferencePoints = ValidReferencePoints + 1
            Else
                ReferencePoint(LinePointer - 9).Valid = ReferencePoint(LinePointer - 9).Valid + 1
            End If

        Next

        If ValidReferencePoints < 3 Then
            If ValidVertexPoints >= 3 Then
                ExtractDataBox.Text = ExtractDataBox.Text & ValidReferencePoints & " usable reference points were found (must be at least 3). " &
                    VertexPointNoted & " boundary points will be used for calibration." & EndLine
            Else
                ExtractDataBox.Text = ExtractDataBox.Text & "Calibration isn't possible, neither with reference points nor with boundary points." & EndLine
            End If
        Else
            ExtractDataBox.Text = ExtractDataBox.Text & "Calibration is possible with  " & ValidReferencePoints & " reference points." & EndLine
        End If

        '- Retrieving value for Dotsize (ds)
        For LinePointer = 41 To GeoDataLineCount
            SplitLine(GeoDataLine(LinePointer))
            If LCase(GeoDataLineField(1)) = "mm1b" Then
                JprData.ds = Val(GeoDataLineField(2))
                ExtractDataBox.Text = ExtractDataBox.Text & "ds (dotsize) = " & CommaToDot(Math.Round(JprData.ds, 5)) & " (Not a JPR-file variable)" & EndLine

            End If

        Next

        '- Retrieving datum (dm) and datum shift (sn and st) from GeoDataLine(5)
        SplitLine(GeoDataLine(5))
        JprData.dm = Trim(LCase(GeoDataLineField(1)))
        JprData.dm = TranslateDatum(JprData.dm)
        JprData.st = Trim(GeoDataLineField(3))
        JprData.sn = Trim(GeoDataLineField(4))

        ExtractDataBox.Text = ExtractDataBox.Text & "dm (datum) = " & JprData.dm & EndLine
        ExtractDataBox.Text = ExtractDataBox.Text & "st (datum shift latitude) = " & JprData.st & EndLine
        ExtractDataBox.Text = ExtractDataBox.Text & "sn (datum shift longitude) = " & JprData.sn & EndLine

        '- Retrieving Utm zone (zn) from GeoDataLine(10)
        SplitLine(GeoDataLine(10))
        JprData.zn = Trim(GeoDataLineField(14))

        '- Settin for North (t) or South (j) based on boundary Point(1) (most releable option)
        If JprData.zn <> "" Then
            If VertexPoint(1).y > 0 Then
                JprData.zn = JprData.zn & "t"
            Else
                JprData.zn = JprData.zn & "j"
            End If
        End If

        '- Retrieving projection (pr) from GeoDataLine(10)
        SplitLine(GeoDataLine(9))
        JprData.pr = LCase(GeoDataLineField(2))
        JprData.pr = TranslateProjection(JprData.pr, JprData.zn)

        '- Retriecing projection specific data from GeoDataLine(40)
        SplitLine(GeoDataLine(40))
        JprData.pp = Trim(GeoDataLineField(3))     'Central meridian of the Chart
        JprData.p1 = Trim(GeoDataLineField(2))     'Latitude of the origin of the Chart
        JprData.p2 = Trim(GeoDataLineField(4))     'Scalefactor of the Chart
        JprData.p3 = Trim(GeoDataLineField(6))     'False Easting of the Chart
        JprData.p4 = Trim(GeoDataLineField(5))     'False Northing of the Chart
        JprData.p5 = Trim(GeoDataLineField(7))     'First standard parallel of the Chart
        JprData.p6 = Trim(GeoDataLineField(8))     'Second standard parallel of the Chart

        '- Writing JprData tov ExtractDataBox
        Select Case JprData.pr
            Case "mercator"
                ExtractDataBox.Text = ExtractDataBox.Text & "pr (projection) = " & JprData.pr & EndLine &
                    "p1 (latitude of the origin) = " & JprData.p1 & EndLine
                If JprData.p1 = "" Then
                    ExtractDataBox.Text = ExtractDataBox.Text & "The projection data are incomplete." & EndLine
                End If

            Case "transverse mercator"
                ExtractDataBox.Text = ExtractDataBox.Text & "pr (projection) = " & JprData.pr & EndLine &
                    "pp (central meridian) = " & JprData.pp & EndLine &
                    "p1 (latitude of the origin) = " & JprData.p1 & EndLine &
                    "p2 (Scalefactor) = " & JprData.p2 & EndLine &
                    "p3 (False Easting) = " & JprData.p3 & EndLine &
                    "p4 (False Northing) = " & JprData.p4 & EndLine
                If JprData.pp = "" Or JprData.p1 = "" Or JprData.p1 = "" Or JprData.p2 = "" Or JprData.p3 = "" Or JprData.p4 = "" Then
                    ExtractDataBox.Text = ExtractDataBox.Text & "The projection data are incomplete." & EndLine
                End If

            Case "utm"
                ExtractDataBox.Text = ExtractDataBox.Text & "pr (projection) = " & JprData.pr & EndLine &
                    "zn (zone) = " & JprData.zn & EndLine
                If JprData.zn = "" Then
                    ExtractDataBox.Text = ExtractDataBox.Text & "The projection data are incomplete" & EndLine
                End If

            Case "lambert conformal conic"
                ExtractDataBox.Text = ExtractDataBox.Text & "pr (projection) = " & JprData.pr & EndLine &
                    "pp (central meridian) = " & JprData.pp & EndLine &
                    "p1 (latitude of the origin) = " & JprData.p1 & EndLine &
                    "p5 (First standard parallel) = " & JprData.p5 & EndLine &
                    "p6 (Second standard parallel) = " & JprData.p6 & EndLine
                If JprData.pp = "" Or JprData.p1 = "" Or JprData.p5 = "" Or JprData.p6 = "" Then
                    ExtractDataBox.Text = ExtractDataBox.Text & "The projection data are incomplete" & EndLine
                End If

            Case Else
                ExtractDataBox.Text = ExtractDataBox.Text & "pr (projection) = " & JprData.pr & ". This projection is not supported." & EndLine &
                    "pp (central meridian) = " & JprData.pp & EndLine &
                    "p1 (latitude of the origin) = " & JprData.p1 & EndLine &
                    "p2 (Scalefactor) = " & JprData.p2 & EndLine &
                    "p3 (False Easting) = " & JprData.p3 & EndLine &
                    "p4 (False Northing) = " & JprData.p4 & EndLine &
                    "p5 (First standard parallel) = " & JprData.p5 & EndLine &
                    "p6 (Second standard parallel) = " & JprData.p6 & EndLine
        End Select

        '- Enabling next steps in the application and moving text cursor to ExtractDataBox
        ExtractGroup.Enabled = True
        ExtractDataBox.Select()
        ExtractDataBox.DeselectAll()
        BuildItem.Enabled = True
        BuildItem.Select()

    End Sub

    'Subroutine BuildItem_Click builds the correct content for the JPR-file
    Private Sub BuildItem_Click(sender As Object, e As EventArgs) Handles BuildItem.Click

        '- Building Content for JPR-File bij using function BuildJprContent
        JprDataBox.Text = BuildJprContent("ini")

        '- Enabling next step in the application and moving text cursor to DataBox
        JprGroup.Enabled = True
        JprDataBox.Select()
        JprDataBox.DeselectAll()
        SaveItem.Enabled = True
        SaveItem.Select()

    End Sub

    'Function BuildJprContent generates the content for the JPR-file (as text string)
    '- Trigger is used to make teh difference between the initiator if teh function
    Private Function BuildJprContent(Trigger As String)
        '- Declarations
        Dim TmpPointer1, TmpPointer2 As Integer     'Subroutine internal variables; represent array pointers
        Dim ContentText As String                   'Subroutine internal variable; represents the content for the JPR-File

        '- Adding data line by line to content
        ContentText = "//This file was created using M4MM " & Version & " (Maps for Memory Map) by www.hzns.nl" & EndLine &
            "nm=" & JprData.nm & EndLine &
            "dm=" & JprData.dm & EndLine &
            "st=" & JprData.st & EndLine &
            "sn=" & JprData.sn & EndLine &
            "pr=" & JprData.pr & EndLine

        '- Handling projection specific variables
        Select Case JprData.pr
            Case "utm"
                ContentText = ContentText & "zn=" & JprData.zn & EndLine
            Case "mercator"
                ContentText = ContentText & "p1=" & JprData.p1 & EndLine
            Case "transverse mercator"
                ContentText = ContentText & "pp=" & JprData.pp & EndLine &
                    "p1=" & JprData.p1 & EndLine &
                    "p2=" & JprData.p2 & EndLine &
                    "p3=" & JprData.p3 & EndLine &
                    "p4=" & JprData.p4 & EndLine
            Case "lambert conformal conic"
                ContentText = ContentText & "pp=" & JprData.pp & EndLine &
                    "p1=" & JprData.p1 & EndLine &
                    "p5=" & JprData.p5 & EndLine &
                    "p6=" & JprData.p6 & EndLine
            Case Else
                ContentText = ContentText & "//This projection is not supported by Maps for Memory-Map (M4MM), please check en edit 'p-data'" & EndLine &
                    "pp=" & JprData.pp & EndLine &
                    "p1=" & JprData.p1 & EndLine &
                    "p2=" & JprData.p2 & EndLine &
                    "p3=" & JprData.p3 & EndLine &
                    "p4=" & JprData.p4 & EndLine &
                    "p5=" & JprData.p5 & EndLine &
                    "p6=" & JprData.p6 & EndLine
        End Select

        '- Handling resolution and scale
        Select Case Trigger
            Case "ini"
                JprData.sr = Val(DpiBox.Text)
                JprData.sc = Int((JprData.sr / 2.54) * 100 * JprData.ds)

                'Writing dat to ScaleBox
                ScaleBox.Text = JprData.sc

            Case "dpi"
                JprData.sr = Val(DpiBox.Text)
                JprData.sc = Int((JprData.sr / 2.54) * 100 * JprData.ds)

                'Writing dat to ScaleBox
                ScaleBox.Text = JprData.sc

            Case "scale"
                JprData.sc = Val(ScaleBox.Text)
                JprData.sr = Int(JprData.sc * 2.54 / (JprData.ds * 100))

                'Writing dat to DpiBox
                DpiBox.Text = JprData.sr

        End Select

        ContentText = ContentText & "sr=" & JprData.sr & EndLine
        ContentText = ContentText & "sc=1:" & JprData.sc & EndLine

        '- Handling graphic variables
        If JprData.it <> "png" Then
            ContentText = ContentText & "You are advised to convert the image to PNG and edit this JPR file (it=png)." & EndLine
        End If
        ContentText = ContentText & "it=" & JprData.it & EndLine

        '- Handling Referrence points
        If ValidVertexPoints < 3 And ValidReferencePoints < 3 Then
            ContentText = ContentText & "//The chart can not calibrated because there aren’t enough points available." & EndLine
        Else
            If ValidVertexPoints > ValidReferencePoints Then
                ContentText = ContentText & "//The boundary points were use for calibration." & EndLine
                TmpPointer1 = 0
                For TmpPointer2 = 1 To VertexPointNoted
                    If VertexPoint(TmpPointer2).Valid = 0 Then
                        TmpPointer1 = TmpPointer1 + 1
                        ContentText = ContentText & "rp" & TmpPointer1 & "=" &
                        CommaToDot(VertexPoint(TmpPointer1).lat) & "," &
                        CommaToDot(VertexPoint(TmpPointer1).lon) & "," &
                        VertexPoint(TmpPointer1).x & "," &
                        VertexPoint(TmpPointer1).y & EndLine
                    End If
                Next
            Else
                ContentText = ContentText & "//The reference points were use for calibration." & EndLine
                TmpPointer1 = 0
                For TmpPointer2 = 1 To ReferencePointCounter
                    If ReferencePoint(TmpPointer2).Valid = 0 Then
                        TmpPointer1 = TmpPointer1 + 1
                        ContentText = ContentText & "rp" & TmpPointer1 & "=" &
                        CommaToDot(ReferencePoint(TmpPointer1).lat) & "," &
                        CommaToDot(ReferencePoint(TmpPointer1).lon) & "," &
                        ReferencePoint(TmpPointer1).x & "," &
                        ReferencePoint(TmpPointer1).y & EndLine
                    End If
                Next
            End If
        End If

        '- Handling vertex polygon points
        If ValidVertexPoints < 3 Then
            ContentText = ContentText & "//The boundary couldn't established because there aren’t enough points available." & EndLine
        Else
            TmpPointer1 = 0
            For TmpPointer2 = 1 To VertexPointNoted
                If VertexPoint(TmpPointer2).Valid = 0 Then
                    TmpPointer1 = TmpPointer1 + 1
                    ContentText = ContentText & "vp" & TmpPointer1 & "=" &
                    VertexPoint(TmpPointer1).x & "," &
                    VertexPoint(TmpPointer1).y & EndLine
                End If
            Next
        End If

        '- Setting return value
        Return ContentText

    End Function

    'Subroutine SaveItem_Click 
    '-  Saving the JrpDataBox.text to a JPR-file in the same directory as the orginal MAP file. 
    '   If this JPR file exists a warning will be prompted.
    Private Sub SaveItem_Click(sender As Object, e As EventArgs) Handles SaveItem.Click
        'Declarations
        Dim Answer As Integer       'Used as answer value for MsgBox

        '- Checking for an existing file
        If My.Computer.FileSystem.FileExists(JprFile.LongName) = True Then
            '  (Msgbox appearance: 4 = Yes/No, 32 = Question mark, 256 = second button default, 292 = total)
            Answer = MsgBox("JPR-file allready exits. Overwrite?", 292, "Warning")
            If Answer = 6 Then
                'Overwriting existing JPR-file if answer on MsgBox = Yes (Answer = 6)
                My.Computer.FileSystem.WriteAllText(JprFile.LongName, JprDataBox.Text, False)
            End If
        Else
            '- Creating JPR-file
            My.Computer.FileSystem.WriteAllText(JprFile.LongName, JprDataBox.Text, False)
        End If

    End Sub

    'Subroutine DipBox_LostFocus handles the cases LostFocus and SelectedValueChanged (with focus on DpiBox)
    '- recalculating the map scale
    Private Sub DpiBox_LostFocus(sender As Object, e As EventArgs) Handles DpiBox.LostFocus, DpiBox.SelectedValueChanged
        '(only if JprDataBox contains information)
        If JprDataBox.Text <> "" Then
            If JprData.sr <> Val(DpiBox.Text) Then
                JprDataBox.Text = BuildJprContent("dpi")
                SaveItem.Select()

            End If
        End If

    End Sub

    'Subroutine DipBox_PressKey handles the case "Enter"-key was used (with focus on DpiBox)
    '- recalculaing the map scale
    Private Sub DpiBox_KeyPress(sender As Object, e As KeyPressEventArgs) Handles DpiBox.KeyPress
        If JprDataBox.Text <> "" And e.KeyChar = Chr(13) Then
            If JprData.sr <> Val(DpiBox.Text) Then
                JprDataBox.Text = BuildJprContent("dpi")
                SaveItem.Select()

            End If
        End If

    End Sub

    'Subroutine ScaleBox_LostFocus handles the cases LostFocus and SelectedValueChanged (with focus on ScaleBox)
    '- recalculating the map resolution
    Private Sub ScaleBox_LostFocus(sender As Object, e As EventArgs) Handles ScaleBox.LostFocus, ScaleBox.SelectedValueChanged
        '(only if JprFileContentBox contains information)
        If JprDataBox.Text <> "" Then
            If JprData.sc <> Val(ScaleBox.Text) Then
                JprDataBox.Text = BuildJprContent("scale")
                SaveItem.Select()
            End If
        End If

    End Sub

    'Subroutine ScaleBox_PressKey handles the case "Enter"-key was used (with focus on ScaleBox)
    '- recalculating the map resolution
    Private Sub ScaleBox_KeyPress(sender As Object, e As KeyPressEventArgs) Handles ScaleBox.KeyPress
        If JprDataBox.Text <> "" And e.KeyChar = Chr(13) Then
            If JprData.sc <> Val(ScaleBox.Text) Then
                JprDataBox.Text = BuildJprContent("scale")
                SaveItem.Select()

            End If
        End If

    End Sub

    'Subroutine SplitLine splits a line with comma separeted fields into max 20 fields
    Private Sub SplitLine(LineText As String)
        '- Declarations
        Dim FieldPointer As Integer     'Variable; represents order of a field in a line
        Dim ChrPointer As Integer       'Variable; represents the place of a character in a string
        Dim TextChr As String           'Variable; represents a character

        '- Clear all information Array GeoDataLineField
        For FieldPointer = 0 To 20
            GeoDataLineField(FieldPointer) = ""
        Next

        '- Splitting line into fields (comma is separator: Chr=44)
        '  and writing to GeoDataLineField array
        FieldPointer = 1
        ChrPointer = 1
        Do While ChrPointer <= Len(LineText)
            Do
                TextChr = Mid(LineText, ChrPointer, 1)
                If TextChr <> Chr(44) Then
                    GeoDataLineField(FieldPointer) = GeoDataLineField(FieldPointer) + TextChr
                End If
                ChrPointer = ChrPointer + 1
            Loop Until TextChr = Chr(44) Or ChrPointer > Len(LineText)
            If TextChr = Chr(44) Then FieldPointer = FieldPointer + 1
        Loop
    End Sub


    'Subroutine PrintDataItem_Click
    '- Initiates Printing GeoDataBox.text with color data
    Private Sub PrintDataItem_Click(sender As Object, e As EventArgs) Handles PrintDataItem.Click

        StartMenu.PrintContent(GeoDataBox.Text, GeoDataBox.Font, LandscapeItem.Checked)

    End Sub

    'Subroutine ExtractedDataItem_Click
    '- Initiates Printing ExtractDataBox.text
    Private Sub ExtractedDataItem_Click(sender As Object, e As EventArgs) Handles ExtractedDataItem.Click

        StartMenu.PrintContent(ExtractDataBox.Text, GeoDataBox.Font, LandscapeItem.Checked)

    End Sub

    'Subroutine JprDataItem_Click
    '- Initiates Printing JprDataBox.text
    Private Sub JprDataItem_Click(sender As Object, e As EventArgs) Handles JprDataItem.Click

        StartMenu.PrintContent(JprDataBox.Text, GeoDataBox.Font, LandscapeItem.Checked)

    End Sub

    'Subroutine PrintItem_Click
    '- Testing if there is something to print (ExtractDataBox and JprDataBox)
    Private Sub PrintItem_Click(sender As Object, e As EventArgs) Handles PrintItem.Click

        '- Enabling/disabling printing ExtractDataBox
        If Trim(ExtractDataBox.Text) = "" Then
            ExtractedDataItem.Enabled = False
        Else
            ExtractedDataItem.Enabled = True
        End If

        '- Enabling/disabling printing JprDataBox
        If Trim(JprDataBox.Text) = "" Then
            JprDataItem.Enabled = False
        Else
            JprDataItem.Enabled = True
        End If

    End Sub

End Class