﻿Imports System.ComponentModel

Public Class SidInfo
    'Declarations
    Public SourceFile As FileNameStructure          'Variable; represents the source-filename (which contains WKT text)

    'Subroutine SidInfo_Load
    '- Actions at opening the form GeoPdfInfo
    Private Sub SidInfo_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        '- Declarations
        Dim TipText As New ToolTip()                'Creating tooltips for this form

        '- Clearing all values in form
        FileValue.Text = ""
        GeoDataBox.Text = ""
        GdalVersion.Text = ""

        '- Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        '- Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        '- Tooltips
        TipText.SetToolTip(TopStrip, "Select a GeoPDF-file to view the calibration information.")
        TipText.SetToolTip(GeoDataBox, "Calibration information (WKT) in MrSID-file.")

        '- Setting form title
        Me.Text = AppName & " - MrSID-file information"

        '- Disabling M4MM form
        StartMenu.Enabled = False

        '- Disabling Print menu items
        PrintItem.Enabled = False

        '- Initiating SelectItem
        SelectItem_Click(sender, e)

    End Sub

    'Subroutine CloseItem_Click
    '- initializing closing the form SidInfo
    Private Sub CloseItem_Click(sender As Object, e As EventArgs) Handles CloseItem.Click
        Me.Close()

    End Sub

    'Subroutine SidInfo_Closing
    '- Actions at closing the form SidInfo
    Private Sub SidInfo_Closing(sender As Object, e As CancelEventArgs) Handles Me.Closing
        '- Enabling StartMenu form
        StartMenu.Enabled = True

    End Sub

    'Subroutine SelectItem_Click
    '- selecting a MrSID-file
    Private Sub SelectItem_Click(sender As Object, e As EventArgs) Handles SelectItem.Click
        '- Declarations

        '- Clearing all values in form
        FileValue.Text = ""
        GeoDataBox.Text = ""
        GdalVersion.Text = ""

        '- Setting patameters for SelectFile
        With SelectFile
            .Filter = "MrSID-file (*.sid)|*.sid"
            .Multiselect = False
        End With

        '- Triggering SelectFile and handling case no file was selected
        If SelectFile.ShowDialog <> DialogResult.OK Then

            FileValue.Text = ""
            MsgBox("No file was selected!", 48, AppName & " - Warning")

            '- Disabling Print menu items
            PrintItem.Enabled = False

            Exit Sub

        End If

        SourceFile = SplitFileName(SelectFile.FileName)

        '- Showing selected filename in form (FileValue)
        FileValue.Text = SourceFile.LongName

        '- Retrieving Geo-information (WKT 2.0 with GdalInfo and writing these data to GdalInfoBox
        GeoDataBox.Text = WktText(SourceFile.LongName)

        '- Retrieving used GDAL version using function UsedGdal
        GdalVersion.Text = "Used GDAL version: " & UsedGdal()

        '- Ensabling StartMenu form
        PrintItem.Enabled = True

    End Sub

    'Subroutine PrintDataItem_Click
    '- Initiates Printing GeoDataBox.text with color data
    Private Sub PrintDataItem_Click(sender As Object, e As EventArgs) Handles PrintDataItem.Click

        StartMenu.PrintContent(GeoDataBox.Text, GeoDataBox.Font, LandscapeItem.Checked)

    End Sub

    'Subroutine PrintDataNoClrItem_Click
    '- Initiates Printing GeoDataBox.text without color data
    Private Sub PrintDataNoClrItem_Click(sender As Object, e As EventArgs) Handles PrintDataNoClrItem.Click

        '- Trimming color data by
        '  splitting into lines 
        '  cycle through lines until "band "  
        Dim TempLine() As String = Split(GeoDataBox.Text, EndLine)
        Dim TempPointer As Integer
        Dim TempText As String = ""

        For TempPointer = 0 To TempLine.Count - 1
            If LCase(Strings.Left(TempLine(TempPointer), 5)) <> "band " Then
                TempText = TempText & TempLine(TempPointer) & EndLine
            Else
                Exit For
            End If
        Next

        StartMenu.PrintContent(TempText, GeoDataBox.Font, LandscapeItem.Checked)

    End Sub

End Class