﻿Public Class StartMenu

    'Variable declarations
    Public IniFile As FileNameStructure             'Variable represents path and Filename to 'm4mm.ini'.
    Public IniContent As String                     'Variable represents content of 'm4mm.ini'.
    Public PrintFont As Font                        'Variable representing the font to be used with printing
    Public ContentString As String                  'Variabel representing a text string to be printed

    'Subroutine M4MM_Load will be executed on starting the application form.
    '- It enables and sets the tooltips for this form.
    '- It shows the disclaimer
    '- It enable/disable menu items
    '- It calls sub 'CheckGdalPath' (to load 'BatFilePath').
    Private Sub M4MM_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        '- Declarations
        Dim TipText As New ToolTip()                'Creating tooltips for this form

        '- Setting up the delays for the ToolTip (TipText).
        TipText.AutoPopDelay = 5000
        TipText.InitialDelay = 1000
        TipText.ReshowDelay = 500

        '- Forcing the ToolTip text to be displayed whether or not the form is active.
        TipText.ShowAlways = True

        TipText.SetToolTip(TopStrip, "Select a menu item to start a functionality.")

        '- Setting form title
        Me.Text = AppName & VersionName

        '- Initial setting for StartMenu
        TopStrip.Enabled = True
        IniItem.Enabled = True
        EndItem.Enabled = True
        InfoPdfItem.Enabled = False
        InfoTifItem.Enabled = False
        InfoOziItem.Enabled = True
        InfoImageItem.Enabled = True
        Map2JprItem.Enabled = True
        Mobac2JprItem.Enabled = True
        InfoJprItem.Enabled = True
        InfoPdfItem.Enabled = False
        InfoTifItem.Enabled = False
        Pdf2JprItem.Enabled = False
        Tiff2JprItem.Enabled = False
        ConvOziItem.Enabled = False
        InfoMrSidItem.Enabled = False
        ConvMrSidItem.Enabled = False

        '- Setting Disclaimer text
        Disclaimer.Text = "This application is provided 'As is'. The use of the application is on your own risk. " &
            "Direct or indirect damage by using this application is users responsibility/risk, " &
            "not the application-builders. Redistribution by a third party (commercial of non-commercial) " &
            "is prohibit. Download the application direct from https://www.hzns.nl." & EndLine & EndLine &
            "This application uses GDAL/OGR which licensed under an MIT/X style license with the following terms:" & EndLine &
            "Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation " &
            "files (the 'Software'), to deal in the Software without restriction, including without limitation the rights to use, copy, " &
            "modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software " &
            "is furnished to do so, subject to the following conditions:" & EndLine & EndLine &
            "THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES " &
            "OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE " &
            "LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN " &
            "CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE." & EndLine

        '- Initializing CheckGdalPath
        Call CheckGdalPath()

    End Sub

    'Subroutine CheckGdalPath 
    '- 'CheckGdalPath' will first look for the 'm4mm.ini' file in documents directory of user.
    '- If so, it will check the content of 'm4mm.ini'. the value of 'BatFilePath' must be a exiting file 'OSGeo4W.bat'.
    '  OSGeo4W.bat starts the OSGeo4W shell. This shell is needed to exceute gdalinfo.exe. 
    '  If found this information Is showed In the application form('BatFileValue'). and the button 'FindButton' will be enabled. 
    Private Sub CheckGdalPath()
        '- Declarations
        Dim Answer As Integer

        '- Looking of m4mm.ini
        '  (My.Computer.FileSystem.SpecialDirectories.MyDocuments points to users 'My documents' directory
        IniFile = SplitFileName(My.Computer.FileSystem.SpecialDirectories.MyDocuments & "\m4mm.ini")
        If My.Computer.FileSystem.FileExists(IniFile.LongName) = True Then
            IniContent = My.Computer.FileSystem.ReadAllText(IniFile.LongName)
            'ini-data is ony one line; text will be split in two parts (part(0) is "OSGeo4Win" and the part(1) is the value for BatFile
            'Before spliting there will be a check if the separating chr (=) exist
            Answer = InStr(IniContent, "=", CompareMethod.Binary)
            If Answer > 0 Then
                Dim Part() As String = Split(IniContent, "=",,)
                If My.Computer.FileSystem.FileExists(Part(1)) = True Then
                    BatFile = SplitFileName(Part(1))    'BatFile = locadtion of OSGeo4W.bat to start OSGeo for windows environment

                    'Enabling application OSGeo for Windows related menu items
                    OsGeoPath.Text = "Used to start OSGeo for Windows: " & BatFile.LongName
                    InfoPdfItem.Enabled = True
                    InfoTifItem.Enabled = True
                    Pdf2JprItem.Enabled = True
                    Tiff2JprItem.Enabled = True
                    ConvOziItem.Enabled = True
                    InfoMrSidItem.Enabled = True
                    ConvMrSidItem.Enabled = True

                    Exit Sub
                Else
                    'Disabling application OSGeo for Windows related menu items
                    OsGeoPath.Text = "OSGeo for Windows is not avaiable. ( to activate: File > Find OSGeo4W.bat)"
                    InfoPdfItem.Enabled = False
                    InfoTifItem.Enabled = False
                    Pdf2JprItem.Enabled = False
                    Tiff2JprItem.Enabled = False
                    ConvOziItem.Enabled = False
                    InfoMrSidItem.Enabled = False
                    ConvMrSidItem.Enabled = False

                End If

            End If

        End If

    End Sub

    'Subroutine FindChangeOS
    '- This sub launches the OpenFileDialog to find the file 'OSGeo4W.bat'.
    '- If found this information Is showed In the application form('BatFileValue'). and the button 'FindButton' will be enabled. 
    '- If not found there will be a warning. No chances will be made ('BatFileValue' and 'FindButton').
    Public Sub FindChangeOS()
        '- Declarations
        Dim TempFile As FileNameStructure
        Dim TempText As String
        Dim SelectFile As New OpenFileDialog

        '- Setting variables for Filedialog
        SelectFile.InitialDirectory = "c:\"
        SelectFile.Filter = "OSGeo4W-file | OSGeo4W.bat"
        SelectFile.Multiselect = False

        '- Opening dialog
        SelectFile.ShowDialog()
        TempFile = SplitFileName(SelectFile.FileName)

        '- Stopping subroutine if OSGeo4W.bat isn't selected (len TempfileName = 0)
        If Len(TempFile.LongName) = 0 Then
            TempText = "No OSGeo4W.bat was selected. Without this file/information the application will not work correctly!"
            MsgBox(TempText, 48, "Warning")
            Exit Sub
        End If

        '- Setting variables
        BatFile = SplitFileName(TempFile.LongName)

        '- Deleting existing m4mm.ini if exists
        If My.Computer.FileSystem.FileExists(IniFile.LongName) = True Then My.Computer.FileSystem.DeleteFile(IniFile.LongName)

        '- Creating text for new m4mm.ini
        TempText = "OSGeo4Win=" & BatFile.LongName
        My.Computer.FileSystem.WriteAllText(IniFile.LongName, TempText, False)

        '- Enabling application OSGeo for Windows related menu items
        OsGeoPath.Text = "Used to start OSGeo for Windows: " & BatFile.LongName
        InfoPdfItem.Enabled = True
        InfoTifItem.Enabled = True
        InfoMrSidItem.Enabled = True
        Pdf2JprItem.Enabled = True
        Tiff2JprItem.Enabled = True
        MrSid2JprItem.Enabled = True

    End Sub

    'Subroutine IniItem_Click
    '- This sub call 'FindChangeOS'.
    '- Enables the oppertunity to change OSGeo4W
    Private Sub IniItem_Click(sender As Object, e As EventArgs) Handles IniItem.Click
        FindChangeOS()

    End Sub

    'Subroutine EndItem_Click
    '- Unloading and closing application
    Private Sub EndItem_Click(sender As Object, e As EventArgs) Handles EndItem.Click
        Me.Close()

    End Sub

    'Subroutine InofPdfItem_click
    '- Initializing form GeoPdfInfo
    Private Sub InfoPdfItem_Click(sender As Object, e As EventArgs) Handles InfoPdfItem.Click
        PdfInfo.Show()

    End Sub

    'Subroutine InfoTifItem_click
    '- Initializing form GeoTifInfo
    Private Sub InfoTifItem_Click(sender As Object, e As EventArgs) Handles InfoTifItem.Click
        TiffInfo.Show()

    End Sub

    'Subroutine InfoMrSidItem_Click
    '- Initializing form SIDInfo
    Private Sub InfoMrSidItem_Click(sender As Object, e As EventArgs) Handles InfoMrSidItem.Click
        SidInfo.Show()

    End Sub

    'Subroutine InfoOziItem
    '- Initializing form OziInfo
    Private Sub InfoOziItem_Click(sender As Object, e As EventArgs) Handles InfoOziItem.Click
        OziInfo.Show()

    End Sub

    'Subroutine InfoImageItem
    '- Initializing form ImageInfo
    Private Sub InfoImageItem_Click(sender As Object, e As EventArgs) Handles InfoImageItem.Click
        ImageInfo.Show()

    End Sub

    'Subroutine InfoJprItem_Click
    '- Initializing form JprItem
    Private Sub InfoJprItem_Click(sender As Object, e As EventArgs) Handles InfoJprItem.Click
        JprInfo.Show()

    End Sub

    'Subroutine Pdf2JprItem_Click
    '- Initializing form PdfBuild
    Private Sub Pdf2JprItem_Click(sender As Object, e As EventArgs) Handles Pdf2JprItem.Click
        PdfBuild.Show()

    End Sub

    'Subroutine Tiff2JprItem_Click
    '- Initializing form TiffBuild
    Private Sub Tiff2JprItem_Click(sender As Object, e As EventArgs) Handles Tiff2JprItem.Click
        TiffBuild.Show()

    End Sub

    'Subroutine MrSid2JprItem_Click
    '- Initializing form MrSidBuild
    Private Sub MrSid2JprItem_Click(sender As Object, e As EventArgs) Handles MrSid2JprItem.Click
        SidBuild.Show()

    End Sub

    'Subroutine Map2JprItem_Click
    '- Initializing form OZiBuild
    Private Sub Map2JprItem_Click(sender As Object, e As EventArgs) Handles Map2JprItem.Click
        OziBuild.Show()

    End Sub

    'Subroutine Mobac2JprItem_Click
    '- Initializing form OZiBuild
    Private Sub Mobac2JprItem_Click(sender As Object, e As EventArgs) Handles Mobac2JprItem.Click
        MobacBuild.Show()

    End Sub

    'Subroutine CalcDpiItem_Click
    '- Initializing form CalcDpi
    Private Sub CalcDpiItem_Click(sender As Object, e As EventArgs) Handles CalcDpiItem.Click
        CalcDpi.Show()

    End Sub

    'Subroutine ConvOziItem_Click
    '- Initializing form ConvOzi
    Private Sub ConvOziItem_Click(sender As Object, e As EventArgs) Handles ConvOziItem.Click
        ConvOzi.Show()

    End Sub

    'Subroutine ConvMrSidItem_Click
    '- Initializing form SidfConv
    Private Sub ConvMrSidItem_Click(sender As Object, e As EventArgs) Handles ConvMrSidItem.Click
        SidConv.Show()

    End Sub

    'Subroutine EditJprItem_Click
    '- Initializing form JprEdit
    Private Sub EditJprItem_Click(sender As Object, e As EventArgs) Handles EditJprItem.Click
        JprEdit.Show()
    End Sub

    'Subroutine PrintContent
    '-  Initiating the printing by selecting a printer and parsing data to PrintString (object
    Public Sub PrintContent(Content As String, ContentFont As Font, LandScape As Boolean)

        '- Setting Landscape value
        PrintString.DefaultPageSettings.Landscape = LandScape

        '- Creating array PageToPrint based on paper orientation
        If LandScape = True Then
            BuildPages(Content, 110, 40)

        Else
            BuildPages(Content, 80, 55)

        End If

        '- Setting the values for SelectPrinter
        SelectPrinter.Document = PrintString
        SelectPrinter.PrinterSettings = PrintString.PrinterSettings
        SelectPrinter.AllowSomePages = True

        '- Startting  DialogToPrint and initiates printing page by page
        If SelectPrinter.ShowDialog() = DialogResult.OK Then

            PrintFont = ContentFont

            For PagePointer = 0 To NumberOfPages
                ContentString = PageToPrint(PagePointer) & EndLine & "page " & PagePointer & EndLine
                PrintString.PrinterSettings = SelectPrinter.PrinterSettings
                PrintString.Print()
            Next

        End If

    End Sub

    'Subroutine PrintString_PrintPage
    '-  Parsing data to PrintString control 
    Private Sub PrintString_PrintPage(sender As Object, e As Printing.PrintPageEventArgs) Handles PrintString.PrintPage

        e.Graphics.DrawString(ContentString, PrintFont, Brushes.Black, 20, 20)

    End Sub

End Class

